=== modified file 'ivle/dispatch/__init__.py'
--- ivle/dispatch/__init__.py	2009-02-03 11:40:40 +0000
+++ ivle/dispatch/__init__.py	2009-02-03 13:02:36 +0000
@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-# Author: Matt Giuca
+# Author: Matt Giuca, Will Grant
 
 """
 This is a mod_python handler program. The correct way to call it is to have
@@ -38,6 +38,7 @@
 
 import mod_python
 from mod_python import apache, Cookie
+import routes
 
 from ivle import util
 import ivle.conf
@@ -48,6 +49,31 @@
 from request import Request
 import plugins.console # XXX: Relies on www/ being in the Python path.
 
+# XXX List of plugins, which will eventually be read in from conf
+# Elements are (module, classname) pairs.
+plugins_HACK = [
+    ('ivle.webapp.admin.user', 'Plugin'),
+]
+
+def get_routes_mapper():
+    """
+    Build a Mapper object for doing URL matching using 'routes', based on the
+    plugins config.
+    """
+    m = routes.Mapper()
+    for plugin_path, classname in plugins_HACK:
+        # Load the plugin module from somewhere in the Python path
+        # (Note that plugin_path is a fully-qualified Python module name).
+        plugin = getattr(__import__(plugin_path, fromlist=[classname]),
+            classname)
+        # Establish a URL pattern for each element of plugin.urls
+        for url in plugin.urls:
+            routex = url[0]
+            view_class = url[1]
+            kwargs_dict = url[2] if len(url) >= 3 else {}
+            m.connect(routex, view=view_class, **kwargs_dict)
+    return m
+
 def handler(req):
     """Handles a request which may be to anywhere in the site except media.
     Intended to be called by mod_python, as a handler.
@@ -88,6 +114,28 @@
     if not req.publicmode:
         req.user = login.get_user_details(req)
 
+    ### BEGIN New plugins framework ###
+    # XXX This should be done ONCE per Python process, not per request.
+    # (Wait till WSGI)
+    # XXX No authentication is done here
+    mapper = get_routes_mapper()
+    matchdict = mapper.match(req.uri)
+    if matchdict is not None:
+        viewcls = matchdict['view']
+        # Get the remaining arguments, less 'view', 'action' and 'controller'
+        # (The latter two seem to be built-in, and we don't want them).
+        kwargs = matchdict.copy()
+        del kwargs['view']
+        del kwargs['action']
+        del kwargs['controller']
+        # Instantiate the view, which should be a BaseView class
+        view = viewcls(req, **kwargs)
+        # Render the output
+        view.render(req)
+        req.store.commit()
+        return req.OK
+    ### END New plugins framework ###
+
     # Check req.app to see if it is valid. 404 if not.
     if req.app is not None and req.app not in ivle.conf.apps.app_url:
         req.throw_error(Request.HTTP_NOT_FOUND,

=== added directory 'ivle/webapp'
=== added file 'ivle/webapp/__init__.py'
=== added directory 'ivle/webapp/admin'
=== added file 'ivle/webapp/admin/__init__.py'
=== added file 'ivle/webapp/admin/user.py'
--- ivle/webapp/admin/user.py	1970-01-01 00:00:00 +0000
+++ ivle/webapp/admin/user.py	2009-02-03 13:02:36 +0000
@@ -0,0 +1,76 @@
+# IVLE - Informatics Virtual Learning Environment
+# Copyright (C) 2007-2009 The University of Melbourne
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+# Author: Matt Giuca, Will Grant
+
+from ivle.webapp.base.views import JSONRESTView
+from ivle.webapp.base.plugins import BasePlugin
+import ivle.database
+import ivle.util
+
+# List of fields returned as part of the user JSON dictionary
+# (as returned by the get_user action)
+user_fields_list = (
+    "login", "state", "unixid", "email", "nick", "fullname",
+    "rolenm", "studentid", "acct_exp", "pass_exp", "last_login",
+    "svn_pass"
+)
+
+class UserRESTView(JSONRESTView):
+    """
+    A REST interface to the user object.
+    """
+    def __init__(self, req, login):
+        super(UserRESTView, self).__init__(self, req, login)
+        self.context = ivle.database.User.get_by_login(req.store, login)
+
+    def GET(self, req):
+        # XXX Check Caps
+        user = ivle.util.object_to_dict(user_fields_list, self.context)
+        # Convert time stamps to nice strings
+        for k in 'pass_exp', 'acct_exp', 'last_login':
+            if user[k] is not None:
+                user[k] = unicode(user[k])
+
+        user['local_password'] = self.context.passhash is not None
+        return user
+
+    def PATCH(self, req, data):
+        # XXX Check Caps
+        # XXX Admins can set extra fields
+        # Note: Cannot change password here (use change_password named op)
+
+        for f in user_fields_list:
+            try:
+                field = data[f]
+                if isinstance(field, str):
+                    field = unicode(field)
+                setattr(self.context, f, field)
+            except KeyError:
+                continue
+
+class Plugin(BasePlugin):
+    """
+    The Plugin class for the user plugin.
+    """
+    # Magic attribute: urls
+    # Sequence of pairs/triples of
+    # (regex str, handler class, kwargs dict)
+    # The kwargs dict is passed to the __init__ of the view object
+    urls = [
+        ('api/user/:login', UserRESTView)
+    ]

=== added directory 'ivle/webapp/base'
=== added file 'ivle/webapp/base/__init__.py'
=== added file 'ivle/webapp/base/plugins.py'
--- ivle/webapp/base/plugins.py	1970-01-01 00:00:00 +0000
+++ ivle/webapp/base/plugins.py	2009-02-03 13:02:36 +0000
@@ -0,0 +1,24 @@
+# IVLE - Informatics Virtual Learning Environment
+# Copyright (C) 2007-2009 The University of Melbourne
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+# Author: Matt Giuca, Will Grant
+
+class BasePlugin(object):
+    """
+    Base class for all Plugin classes.
+    """
+    pass

=== added file 'ivle/webapp/base/views.py'
--- ivle/webapp/base/views.py	1970-01-01 00:00:00 +0000
+++ ivle/webapp/base/views.py	2009-02-03 13:02:36 +0000
@@ -0,0 +1,64 @@
+# IVLE - Informatics Virtual Learning Environment
+# Copyright (C) 2007-2009 The University of Melbourne
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+# Author: Matt Giuca, Will Grant
+
+import cjson
+
+class BaseView(object):
+    """
+    Abstract base class for all view objects.
+    """
+    def __init__(self, req, **kwargs):
+        pass
+    def render(self, req):
+        pass
+
+class RESTView(BaseView):
+    """
+    A view which provides a RESTful interface. The content type is
+    unspecified (see JSONRESTView for a specific content type).
+    """
+    content_type = "application/octet-stream"
+
+    def __init__(self, req, *args, **kwargs):
+        pass
+
+    def render(self, req):
+        if req.method == 'GET':
+            outstr = self.GET(req)
+        # XXX PATCH hack
+        if req.method == 'PUT':
+            outstr = self.PATCH(req, req.read())
+        req.content_type = self.content_type
+        req.write(outstr)
+
+class JSONRESTView(RESTView):
+    """
+    A special case of RESTView which deals entirely in JSON.
+    """
+    content_type = "application/json"
+
+    def render(self, req):
+        if req.method == 'GET':
+            outjson = self.GET(req)
+        # XXX PATCH hack
+        if req.method == 'PUT':
+            outjson = self.PATCH(req, cjson.decode(req.read()))
+        req.content_type = self.content_type
+        req.write(cjson.encode(outjson))
+        req.write("\n")

